/*

  $Id: Settings.h 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4cClientConfig_Settings_h__
#define __Auth4cClientConfig_Settings_h__

#include <Auth4cClientConfig/Auth4cClientConfig.h>

#include <Auth4cData/Exception.h>
#include <Auth4cData/Profile.h>

#include <string>
#include <vector>
#include <map>

#if defined (LINUX)
#	include <librfc/XMLDoc.hpp>
#endif /* LINUX */

namespace Auth4cClientConfig
{
	class Auth4cClientConfig_Export Settings
	{
		public:
			class Auth4cClientConfig_Export Domain
			{
				private:
					std::string _strDomain;

				public:
					Domain( const std::string& strDomain = std::string() );
					Domain( const Domain& domain );

					const Domain& operator=( const Domain& domain );

					const std::string& domain() const;
					bool isDefault() const;

					void setDefault( bool bDefault );
			};

			class DomainMap : public std::map<std::string, Domain>
			{
			};

			class Auth4cClientConfig_Export Profile : public Auth4cData::Profile
			{
				private:

				public:
					Profile( const std::string& strProfile = std::string() );
					Profile( const Profile& profile );
					Profile( const Auth4cData::Profile& profile );

					const Profile& operator=( const Profile& profile );

					bool isDefault() const;

					void setDefault( bool bDefault );
			};

			class ProfileMap : public std::map<std::string, Profile>
			{
			};

		private:
			bool _bAdmin;
#if defined (LINUX)
			std::string _strHKCU;
			std::string _strHKLM;
			Rfc::XMLDoc _docHKCU;
			Rfc::XMLDoc _docHKLM;
			Rfc::XMLNode* _keyHKCU;
			Rfc::XMLNode* _keyHKLM;
			Rfc::XMLNode* _keyRoot4cCU;
			Rfc::XMLNode* _keyRoot4cLM;
			Rfc::XMLNode* _keyRootAuth4cCU;
			Rfc::XMLNode* _keyRootAuth4cLM;
#elif defined (_WINDOWS)
			HKEY _keyRoot4cCU;
			HKEY _keyRoot4cLM;
			HKEY _keyRootAuth4cCU;
			HKEY _keyRootAuth4cLM;
#endif /* LINUX */
			DomainMap _domainS;
			std::string _strDomain;
			ProfileMap _profileS;
			std::string _strProfile;
			std::string _strUserId;
			std::string _strAuth4cLocal;
			std::string _strAuth4cMaster;
			static Settings* _psettings;

		public:
			Settings();
			virtual ~Settings();

			void load( bool bAdmin = false );
			void loadProfiles();
			const DomainMap& domains() const;
			const std::string& defaultDomain() const;
			const ProfileMap& profiles() const;
			const std::string& defaultProfile() const;
			const std::string& userId() const;
			const std::string& auth4cLocal() const;
			const std::string& auth4cMaster() const;

			void setDefaultDomain( const Domain& domain );
			void setDefaultProfile( const Profile& profile );
			void setUserId( const std::string& strUserId );
			void setAuth4cLocal( const std::string& strConnect );
			void setAuth4cMaster( const std::string& strConnect );

#if defined (LINUX)
			void save();
#endif /* LINUX */

		protected:
			friend class Domain;
			friend class Profile;
	};
};

#endif /* !__Settings_h__ */
