/*

  $Id: AuthenticationLibrary.h 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4c_AuthenticationLibrary_h__
#define __Auth4c_AuthenticationLibrary_h__

#include <Auth4cData/Auth4cData.h>
#include <Auth4cData/Authenticator.h>
#include <Auth4cData/Base.h>
#include <Auth4cData/SharedObject.h>
#include <Auth4cData/InitMap.h>

#include <map>

namespace Auth4cData
{
	class Auth4cData_Export AuthenticationLibrary : public Base
	{
		public:
#if defined (LINUX)
			typedef void* Handle;
#elif defined (_WINDOWS)
			typedef HMODULE Handle;
#endif /* LINUX */
			typedef enum { dispositionUnknown = '-',
							dispositionConfig = 'C',
							dispositionHashedPassword = 'H',
							dispositionNotUsed = 'N',
							dispositionClearPassword = 'P',
							dispositionMap = 'X' } AuthenticationTokenDisposition;
			
		private:
			int _nAuthenticationLibrary;
			std::string _strDescription;
			InitMap _initLibrary;
			AuthenticationTokenDisposition _authenticationTokenDisposition;
			bool _bUserTools;
			SharedObjectPtrMap _sharedObjectS;

		protected:
			Handle _handle;

		public:
			AuthenticationLibrary();
			AuthenticationLibrary( const AuthenticationLibrary* ptemplate );
			AuthenticationLibrary( int nAuthenticationLibrary,
									const std::string& strDescription,
									const InitMap& initLibrary,
									AuthenticationTokenDisposition authenticationTokenDisposition,
									bool bUserTools );
			AuthenticationLibrary( const AuthenticationLibrary& library );
			virtual ~AuthenticationLibrary();

			const AuthenticationLibrary& operator=( const AuthenticationLibrary& library );

			AuthenticationLibrary* load();

			int authenticationLibrary() const;
			const std::string& description() const;
			const SharedObject& sharedObject( const std::string& strPlatform = std::string() ) const;
			const InitMap& libraryInit() const;
			AuthenticationTokenDisposition authenticationTokenDisposition() const;
			bool userTools() const;
			const SharedObjectPtrMap& sharedObjects() const;

			void setDescription( const std::string& strDescription );
			void setLibraryInit( const InitMap& initLibrary );
			void setAuthenticationTokenDisposition( AuthenticationTokenDisposition authenticationTokenDisposition );
			void setUserTools( bool bUserTools );
			SharedObjectPtrMap& sharedObjects();

			virtual Authenticator* authenticator( const Domain& domain,
													const std::string& strDescription,
													const InitMap& initAuthenticator );
	};

	class AuthenticationLibraryMap : public std::map<int, AuthenticationLibrary>
	{
	};

	class AuthenticationLibraryPtrMap : public std::map<int, AuthenticationLibrary*>
	{
	};
};

#endif /* !__Auth4c_AuthenticationLibrary_h__ */
