/*

  $Id: Authenticator.h 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4c_Authenticator_h__
#define __Auth4c_Authenticator_h__

#include <Auth4cData/Auth4cData.h>
#include <Auth4cData/Base.h>
#include <Auth4cData/AuthenticatedUser.h>
#include <Auth4cData/AuthenticationToken.h>
#include <Auth4cData/Domain.h>
#include <Auth4cData/Team.h>
#include <Auth4cData/InitMap.h>

#include <map>
#include <string>

namespace Auth4cData
{
	class Auth4cData_Export Authenticator : public Base
	{
		private:
			Domain _domain;
			std::string _strDescription;
			int _nAuthenticationLibrary;
			InitMap _initAuthenticator;
			bool _bUserTools;

		public:
			Authenticator( const InitMap& initAuthenticator,
							const Domain& domain = Domain(),
							const std::string& strDescription = std::string(),
							int nAuthenticationLibrary = 0 );
			Authenticator( const Authenticator& authenticator );
			virtual ~Authenticator();

			const Authenticator& operator=( const Authenticator& authenticator );

			const std::string& description() const;
			int authenticationLibrary() const;
			const InitMap& authenticatorInit() const;
			const Domain& domain() const;
			bool userTools() const;

			void setDescription( const std::string& strDescription );
			void setAuthenticationLibrary( int nAuthenticationLibrary );
			void setAuthenticatorInit( const InitMap& initAuthenticator );
			void setUserTools( bool bUserTools );

			virtual AuthenticatedUser authenticate( const User& user, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile )  = 0;
			virtual void user( User& user, bool bRecurse = true )  = 0;
			virtual void team( Team& team )  = 0;
			virtual void findUsers( UserMap& userS, const std::string strSearch )  = 0;

// Only implemented if user tools are supported, which is only Auth4cAuth4c
			virtual void users( Auth4cData::UserMap& userS ) ;
			virtual void passwd( const Auth4cData::AuthenticatedUser& user, Auth4cData::AuthenticationToken& authTokenCurrent, Auth4cData::AuthenticationToken& authTokenNew ) ;
			virtual void createUser( const Auth4cData::User& user ) ;
			virtual void updateUser( const Auth4cData::User& user ) ;
			virtual void deleteUser( const Auth4cData::User& user ) ;
			virtual void createTeamMember( Auth4cData::Team& team, Auth4cData::User& user ) ;
			virtual void deleteTeamMember( Auth4cData::Team& team, Auth4cData::User& user ) ;
	};

	class AuthenticatorPtrMap : public std::map<std::string, Authenticator*> // maps Domain string to Authenticator*
	{
	};
};

#endif /* !__Auth4c_Authenticator_h__ */
