/*

  $Id: Exception.h 737 2020-09-23 18:25:13Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4c_Exception_h__
#define __Auth4c_Exception_h__

#include <Auth4cData/Auth4cData.h>

#include <string>
#include <iostream>

namespace Auth4cData
{
	// Do NOT use zero. The XML layer checks that the error code is numeric using atoi
	const int nNAX_UNKNOWN = -1;
	const int nNAX_AUTHENTICATIONFAILED = 1;
	const int nNAX_APPLICATIONABORTED = 5;
	const int nNAX_BADREQUEST = 10;
	const int nNAX_RESPONSETIMEOUT = 20;
	const int nNAX_ALIENEXCEPTION = 100;
	const int nNAX_USERNOTFOUND = 101;
	const int nNAX_CUSTOMERNOTFOUND = 102;
	const int nNAX_DOMAINNOTFOUND = 103;
	const int nNAX_TEAMNOTFOUND = 104;
	const int nNAX_PRODUCTNOTFOUND = 105;
	const int nNAX_PLATFORMNOTFOUND = 106;
	const int nNAX_AUTHENTICATIONLIBRARYNOTFOUND = 107;
	const int nNAX_PERMISSIONNOTFOUND = 108;
	const int nNAX_ROLEPERMISSIONNOTFOUND = 109;
	const int nNAX_BADSESSION = 201;
	const int nNAX_NOTEAMLEADSET = 501;
	const int nNAX_NOCUSTOMERSET = 502;
	const int nNAX_NODOMAINSET = 503;
	const int nNAX_NOPLATFORMSET = 504;
	const int nNAX_NOAUTHENTICATIONLIBRARYSET = 505;
	const int nNAX_NOSHAREDOBJECTSET = 506;
	const int nNAX_NOPAIRSET = 520;
	const int nNAX_UPDATEFAILED = 600;
	const int nNAX_MISMATCHEDNODE = 1001;
	const int nNAX_MALFORMEDXML = 1011;
	const int nNAX_DATABASE = 1021;
	const int nNAX_NOSERVERCONNECT = 1022;
	const int nNAX_MISSINGDATA = 2001;
	const int nNAX_BADTEMPLATE = 3001;
	const int nNAX_NOSUCHVARIABLE = 3002;
	const int nNAX_NOSUCHPRODUCT = 3003;
	const int nNAX_NOSUCHPROFILE = 3005;
	const int nNAX_NOSHAREDOBJECT = 5001;
	const int nNAX_NOENTRYPOINT = 5002;
	const int nNAX_LIBRARYNOTLOADED = 5003;
	const int nNAX_AUTHENTICATIONLIBRARYINITFAILED = 5011;
	const int nNAX_AUTHENTICATORINITFAILED = 5012;
	const int nNAX_NOAUTHENTICATORSET = 5021;
	const int nNAX_NOPRODUCTSET = 5022;
	const int nNAX_NOROLESET = 5023;
	const int nNAX_NODEFAULTPROFILESET = 5031;
	const int nNAX_NOUNIQUEUSER = 5101;
	const int nNAX_VERSIONMISMATCH = 5201;
	const int nNAX_KEYOPENFAILED = 6001;
	const int nNAX_CONNECTOPENFAILED = 6002;
//	const int nNAX_NODOCUMENTSET = 6101;
//	const int nNAX_NODISPOSITIONSET = 6102;
//	const int nNAX_DISPOSITIONNOTFOUND = 6103;
//	const int nNAX_DOCPREPFAILED = 6104;
//	const int nNAX_DOCSIGNFAILED = 6105;
//	const int nNAX_BADCHECKSUM = 6106;
//	const int nNAX_BADSIGNATURE = 6107;
	const int nNAX_BADCONFIG = 6111;
	const int nNAX_BADUSERFORMAT = 6151;
	const int nNAX_PERMISSIONDENIED = 6666;
	const int nNAX_NOTIMPLEMENTED = 7001;
	const int nNAX_FILEOPENFAILED = 7100;
	const int nNAX_SEARCHEXECFAILED = 7500;

	class Auth4cData_Export Exception
	{
		private:
			std::string _strFile;
			int _nLine;
			int _nErrorCode;
			std::string _strLegend;
// If not using the XML client, _bRemote will be false and can be ignored.
// If using the XML client, if _bRemote is true, the exception was thrown by the
// server and tranmitted across the wire, otherwise, the exception was generated
// on the client side, most likely because of a failure in communications.
// The only time this should ever be set to true is within the communications layer
// of the XML client.
// Setting this to true anywhere else will lead to confusion and earn you the ire
// of whoever spends weeks trying to work out why a given exception landed in their
// lap, when it should have landed in somebody else's.
			bool _bRemote;

		public:
			Exception();
			Exception( const std::string& strFile, int nLine, int nErrorCode, const std::string& strLegend, bool bRemote = false );
			Exception( const Exception& exception );
			virtual ~Exception();

			const Exception& operator=( const Exception& exception );

			const std::string& file() const;
			int line() const;
			int errorCode() const;
			const std::string& legend() const;
			bool remote() const;

			static std::string explainErrorCode( int nErrorCode );
	};
};

Auth4cData_Export std::ostream& operator<<( std::ostream& stream, const Auth4cData::Exception& e );

#endif /* !__Auth4c_Exception_h__ */
