/*

  $Id: Factory.h 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4cData_Auth4cFactory_h__
#define __Auth4cData_Auth4cFactory_h__

#include <Auth4cData/Auth4cData.h>

#include <Auth4cData/Platform.h>
#include <Auth4cData/SharedObject.h>
#include <Auth4cData/AuthenticationLibrary.h>
#include <Auth4cData/Profile.h>
#include <Auth4cData/Domain.h>
#include <Auth4cData/Product.h>
#include <Auth4cData/Customer.h>
#include <Auth4cData/XTeam.h>
#include <Auth4cData/User.h>
#include <Auth4cData/AuthenticationLibrary.h>
#include <Auth4cData/Authenticator.h>
#include <Auth4cData/AuthenticationToken.h>
#include <Auth4cData/Exporter.h>
#include <Auth4cData/SessionMap.h>

#include <Base4c/Version.h>

#include <string>

namespace Auth4cDataXML
{
	class Factory;
};

namespace Auth4cDataDB
{
	class Factory;
};

namespace Auth4cData
{
	class Auth4cData_Export Factory
	{
		public:
			typedef enum { modeSimple,			// Domains, Customers, Teams, Products, Roles, Permissions
							modeAdmin,			// as Simple plus Users
							modeAuthenticator,	// as Simple plus AuthenticationLibraries and Authenticators
							modeSignoff,		// document sign-off
							modeAll				// Everything
						} Mode;

		private:
			static Auth4cData::Exporter* _pexporter;
			static Auth4cData::ProductMap _productS;
			static Auth4cData::DomainMap _domainS;
			static Auth4cData::ProfileMap _profileS;
			static Auth4cData::CustomerMap _customerS;
			static Auth4cData::TeamMap _teamS;
//			static Auth4cData::UserMap _userS;
			static Auth4cData::XTeamMap _xteamS;
			static Auth4cData::XTeamPtrAlienMap _alienS;
			static Auth4cData::XTeamPtrAlienRevMap _ralienS;
			static Auth4cData::PlatformMap _platformS;
			static Auth4cData::SharedObjectMap _sharedObjectS;
			static Auth4cData::AuthenticationLibraryMap _authenticationLibraryTemplateS;
			static Auth4cData::AuthenticationLibraryPtrMap _authenticationLibraryS;
			static Auth4cData::AuthenticatorPtrMap _authenticatorS;
			static Auth4cData::AuthenticatedUser _userAuth;
			static Auth4cData::SessionMap _sessionS;
			
		public:
			static Auth4cData::ProductMap& products();
			static Auth4cData::DomainMap& domains();
			static Auth4cData::ProfileMap& profiles();
			static Auth4cData::CustomerMap& customers();
			static Auth4cData::TeamMap& teams();
//			static Auth4cData::UserMap& users();
			static Auth4cData::XTeamMap& xteams();
			static Auth4cData::XTeamPtrAlienMap& aliens();
			static Auth4cData::XTeamPtrAlienRevMap& raliens();
			static Auth4cData::PlatformMap& platforms();
			static Auth4cData::SharedObjectMap& sharedObjects();
			static Auth4cData::AuthenticationLibraryMap& authenticationLibraryTemplates();
			static Auth4cData::AuthenticationLibraryPtrMap& authenticationLibraries();
			static Auth4cData::AuthenticatorPtrMap& authenticators();
			
			static Auth4cData::Permission& createRolePermission( Auth4cData::Role& prole, const Auth4cData::Permission& ppermission );
			static Auth4cData::Permission& createPermission( Auth4cData::Product& pproduct, int nPerm );
			static Auth4cData::Role& createRole( Auth4cData::Product& pproduct, int nRole );
			static Auth4cData::Product& createProduct( int nProduct );

			static Auth4cData::AuthenticatedUser& authenticate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile ) ;
			static Auth4cData::AuthenticatedUser& authenticatedUser() ;
			static bool passwd( const Auth4cData::AuthenticatedUser& user, Auth4cData::AuthenticationToken& authTokenCurrent, Auth4cData::AuthenticationToken& authTokenNew ) ;
			static void persistSession( Auth4cData::AuthenticatedUser& userP ) ;
			static Auth4cData::AuthenticatedUser& retrieveSession( const Auth4cData::Session& session ) ;
			static void killSession( const Auth4cData::Session& session ) ;
			static void user( Auth4cData::User& userF, bool bRecurse = true ) ;
			static void team( Auth4cData::Team& team, bool bReload ) ;

			static const XTeamPtrMap& teams( const AlienKey& alien );
			static const XTeamPtrMap& teams( const Team& team );

			static void mergeRolePermissionsIntoPermissions( ProductMap& productS );

			static Auth4cData::Exporter* exporter();
			static void setExporter( Auth4cData::Exporter* pexporter );

			static bool hasProductPermission();

			static Base4c::Version version();

		private:
			static void setAuthenticatedUser( const AuthenticatedUser& user );

			friend class Auth4cDataXML::Factory;
			friend class Auth4cDataDB::Factory;
	};
};

#endif /* !__Auth4cData_Auth4cFactory_h__ */
