/*

  $Id: Permission.h 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4c_Permission_h__
#define __Auth4c_Permission_h__

#include <Auth4cData/Auth4cData.h>
#include <Auth4cData/Base.h>
#include <Auth4cData/Exception.h>
#include <Auth4cData/PRStatus.h>

#include <string>
#include <map>

namespace Auth4cData
{
	class Product;
	class Role;

	class Auth4cData_Export Permission : public Base
	{
		private:
			const Product* _pproduct;
			const Role* _prole;
			int _nPermission;
			std::string _strDescription;
			PRStatus _status;

		public:
			Permission();
			Permission( const Product& product, int nPermission, const std::string& strDescription = std::string(), PRStatus status = PRStatus::statusDenied );
			Permission( const Role& role, int nPermission, const std::string& strDescription = std::string(), PRStatus status = PRStatus::statusDenied );
			Permission( const Permission& permission );
			virtual ~Permission();

			const Permission& operator=( const Permission& permission );

			const Product& product() const;
			bool hasRole();
			const Role& role() const;
			int permission() const;
			const std::string& description() const;
			const PRStatus& status() const;
			bool hasPermission() const;
			PRStatus& status();

			void setDescription( const std::string& strDescription );
			void setStatus( const PRStatus& status );

		protected:
			void resetProduct( const Product& product );

			friend class Product;
	};

	class Auth4cData_Export PermissionMap : public std::map<int, Permission>
	{
		public:
			void merge( const PermissionMap& map );
			void merge( const Permission& permission );
	};
};

#endif /* !__Auth4c_Permission_h__ */
