/*

  $Id: Product.h 607 2017-05-26 13:06:51Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4c_Product_h__
#define __Auth4c_Product_h__

#include <Auth4cData/Auth4cData.h>
#include <Auth4cData/Base.h>
#include <Auth4cData/Role.h>
#include <Auth4cData/Permission.h>
#include <Auth4cData/InitMap.h>

namespace Auth4cData
{
	class Auth4cData_Export Product : public Base
	{
		private:
			int _nProduct;
			std::string _strDescription;
			InitMapMap _initProduct;
			RoleMap _roleS;
			PermissionMap _permissionS;

		public:
			Product( int nProduct = -1 );
			Product( int nProduct,
						const InitMapMap& initProduct,
						const std::string& strDescription = std::string() );
			Product( const Product& product );
			virtual ~Product();

			const Product& operator=( const Product& product );

			int product() const;
			const std::string& description() const;
			const InitMapMap& productInit() const;
			const RoleMap& roles() const;
			const PermissionMap& permissions() const;

			void setDescription( const std::string& strDescription );
			void setProductInit( const InitMapMap& initProduct );
			void updateInitMap( const std::string& strProfile, const std::string& strOldParam, const std::string& strName, const std::string& strValue, bool bAddNew=true );
			RoleMap& roles();
			PermissionMap& permissions();

			bool hasRole( const Role& role ) const;
			bool hasPermission( const Permission& permission ) const;

		protected:
			void resetPR();
	};

	class Auth4cData_Export ProductMap : public std::map<int, Product>
	{
	};

	class Auth4cData_Export ProductInitMap : public std::map<int, InitMap>
	{
	};
};

#endif /* !__Auth4c_Product_h__ */
