/*

  $Id: Role.h 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4c_Role_h__
#define __Auth4c_Role_h__

#include <Auth4cData/Auth4cData.h>
#include <Auth4cData/Base.h>
#include <Auth4cData/Permission.h>
#include <Auth4cData/Exception.h>
#include <Auth4cData/PRStatus.h>

namespace Auth4cData
{
	class Product;

	class Auth4cData_Export Role : public Base
	{
		private:
			const Product* _pproduct;
			int _nRole;
			std::string _strDescription;
			PRStatus _status;
			PermissionMap _permissionS;

		public:
			Role();
			Role( const Product& product, int nRole, const std::string& strDescription = std::string(), PRStatus status = PRStatus::statusDenied );
			Role( const Role& role );
			virtual ~Role();

			const Role& operator=( const Role& role );

			const Product& product() const;
			int role() const;
			const std::string& description() const;
			const PRStatus& status() const;
			bool hasRole() const;
			PRStatus& status();
			const PermissionMap& permissions() const;

			void setDescription( const std::string& strDescription );
			void setStatus( const PRStatus& status );
			PermissionMap& permissions();

		protected:
			void resetProduct( const Product& product );

			friend class Product;
	};

	class Auth4cData_Export RoleMap : public std::map<int, Role>
	{
		public:
			void merge( const RoleMap& map );
			void merge( const Role& role );
	};
};

#endif /* !__Auth4c_Role_h__ */
