/*

  $Id: User.h 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4c_User_h__
#define __Auth4c_User_h__

#include <Auth4cData/Auth4cData.h>
#include <Auth4cData/Base.h>
#include <Auth4cData/Product.h>

namespace Auth4cData
{
	class Team;
	class TeamMap;
	class Domain;

	class Auth4cData_Export User : public Base
	{
		public:
			typedef std::pair<std::string, std::string> Key;

		private:
			const Domain* _pdomain;
			std::string _strUserId;
			std::string _strName;
			std::string _strAuthenticationToken;
			std::string _strContact;
			ProductMap _productS;
			TeamMap* _pteamS;
			Key _key;

		public:
			User();
			User( const Domain& domain,
					const std::string& strUserId,
					const std::string& strName = std::string(),
					const std::string& strAuthenticationToken = std::string(),
					const std::string& strContact = std::string() );
			User( const User& user );
			virtual ~User();

			const User& operator=( const User& user );

			bool operator==( const User& user ) const;

			const Domain& domain() const;
			const std::string& userId() const;
			const std::string& name() const;
			const std::string& authenticationToken() const;
			const std::string& contact() const;
			const ProductMap& products() const;
			const TeamMap& teams() const;
			const Key& key() const;

			void setUserId( const std::string& strUserId );
			void setName( const std::string& strName );
			void setAuthenticationToken( const std::string& strAuthenticationToken );
			void setContact( const std::string& strContact );
			ProductMap& products();
			TeamMap& teams();

			void update();
			void addTeam( Team& team );
			void deleteTeam( Team& team );

			bool hasRole( const Role& role ) const;
			bool hasPermission( const Permission& permission ) const;
			bool isMemberOf( const Team& team ) const;
			bool isMemberOf( const TeamMap& teamS ) const;

			static User fromText( const std::string& strText );
	};

	class Auth4cData_Export UserMap : public std::map<User::Key, User>
	{
	};

	class Auth4cData_Export UserPtrMap : public std::map<User::Key, User*>
	{
	};
};

#endif /* !__Auth4c_User_h__ */
