/*

  $Id: Value.h 387 2015-12-08 18:53:52Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4cData_Value_h__
#define __Auth4cData_Value_h__

#include <Auth4cData/Auth4cData.h>
#include <Auth4cData/Exception.h>

#include <string>
#include <map>

class XMLNode;

namespace Auth4cData
{
	class Base;

	class Auth4cData_Export Value
	{
		private:
			Base* _pbase;
			std::string _strName;

		public:
			Value();
			Value( Base* pbase, const std::string& strName );
			Value( const Value& value );
			virtual ~Value();

			const Value& operator=( const Value& value );

			const std::string& name() const;

			virtual XMLNode* toXML( XMLNode* pnodeAddTo ) const = 0;
			virtual void fromXML( const XMLNode* pnodeSource ) throw ( Exception ) = 0;
	};

	class Auth4cData_Export ValuePtrMap : public std::map<std::string, Value*>
	{
		public:
			virtual void toXML( XMLNode* pnodeAddTo ) const;
	};

	class Auth4cData_Export StringValue : public Value
	{
		private:
			std::string _strValue;

		public:
			StringValue();
			StringValue( Base* pbase, const std::string& strName, const std::string& strValue = std::string() );
			StringValue( const StringValue& value );
			virtual ~StringValue();

			const StringValue& operator=( const StringValue& value );
			const StringValue& operator=( const std::string& strValue );
			const std::string& value() const;
			operator const std::string&() const;

			virtual XMLNode* toXML( XMLNode* pnodeAddTo ) const;
			virtual void fromXML( const XMLNode* pnodeSource ) throw ( Exception );
	};

	class Auth4cData_Export IntValue : public Value
	{
		private:
			int _nValue;

		public:
			IntValue();
			IntValue( Base* pbase, const std::string& strName, int nValue = 0 );
			IntValue( const IntValue& value );
			virtual ~IntValue();

			const IntValue& operator=( const IntValue& value );
			const IntValue& operator=( int nValue );
			int value() const;
			operator int() const;

			virtual XMLNode* toXML( XMLNode* pnodeAddTo ) const;
			virtual void fromXML( const XMLNode* pnodeSource ) throw ( Exception );
	};

	class Collection : public Value
	{
		public:
			Collection();
			Collection( Base* pbase, const std::string& strName );
			Collection( const Collection& collection );
			virtual ~Collection();

			const Collection& operator=( const Collection& collection );

			virtual XMLNode* toXML( XMLNode* pnodeAddTo );
			virtual void fromXML( const XMLNode* pnodeSource ) throw ( Exception );
	};
};

#endif /* !__Auth4cData_Value_h__ */
