/*

  $Id: Auth4cFactory.h 689 2018-11-28 17:54:35Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4cDB_Auth4cFactory_h__
#define __Auth4cDB_Auth4cFactory_h__

#include <Auth4cDataDB/Auth4cDataDB.h>

#include <Auth4cData/Factory.h>

#include <Auth4cData/Product.h>
#include <Auth4cData/User.h>
#include <Auth4cData/AuthenticationLibrary.h>
#include <Auth4cData/Authenticator.h>
#include <Auth4cData/AuthenticationToken.h>

#include <Base4c/Version.h>

#include <librfcdal/DalDatabase.h>

#include <string>

namespace Auth4cDataDB
{
	class Auth4cDataDB_Export Factory : public Auth4cData::Factory
	{
		private:
			static Mode _mode;

		public:
			static void initialize( const std::string& strConnect, Mode mode = modeSimple );
			static void reinitialize();
			static Auth4cData::AuthenticatedUser authenticate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile );
			static void persistSession( Auth4cData::AuthenticatedUser& userP );
			static Auth4cData::AuthenticatedUser& retrieveSession( const Auth4cData::Session& session );
//			static void authorize( Auth4cData::AuthenticatedUser& user );
			static void authorize( Auth4cData::User& user );
			static void user( Auth4cData::User& user, bool bRecurse );

			static Auth4cData::ProfileMap& profiles();
			static Auth4cData::ProfileMap profiles( const std::string& strHost );

		public:
			static Auth4cData::Product& productFromRow( RfcDal::DalDatabase::Row& row );
			static Auth4cData::Domain& domainFromRow( RfcDal::DalDatabase::Row& row );
			static Auth4cData::Profile& profileFromRow( RfcDal::DalDatabase::Row& row );
			static Auth4cData::Profile profileFromRowNI( RfcDal::DalDatabase::Row& row );
			static Auth4cData::Authenticator& authenticatorFromDomainRow( RfcDal::DalDatabase::Row& row );
			static Auth4cData::Customer& customerFromRow( RfcDal::DalDatabase::Row& row );
			static Auth4cData::Team& teamFromRow( RfcDal::DalDatabase::Row& row );
			static void teamMapFromRow( RfcDal::DalDatabase::Row& row );
			static Auth4cData::User& userFromRow( RfcDal::DalDatabase::Row& row );
			static Auth4cData::Platform& platformFromRow( RfcDal::DalDatabase::Row& row );
			static Auth4cData::SharedObject& sharedObjectFromRow( RfcDal::DalDatabase::Row& row );
			static Auth4cData::AuthenticationLibrary& authenticationLibraryFromRow( RfcDal::DalDatabase::Row& row );

			static Auth4cData::RoleMap roleMapFromResult( const Auth4cData::Product& product, const RfcDal::DalDatabase::Result& result );
			static Auth4cData::PermissionMap permissionMapFromResult( const Auth4cData::Product& product, const RfcDal::DalDatabase::Result& result );

			static void productRoleMapFromResult( Auth4cData::ProductMap& productS, const RfcDal::DalDatabase::Result& result );
			static void productPermissionMapFromResult( Auth4cData::ProductMap& productS, const RfcDal::DalDatabase::Result& result );

			static Auth4cData::InitMap initMapFromResult( const RfcDal::DalDatabase::Result& result );
			static Auth4cData::InitMapMap initMapMapFromResult( const RfcDal::DalDatabase::Result& result );

		public:
			static void createCustomer( Auth4cData::Customer& customer );
			static void updateCustomer( Auth4cData::Customer& customer );
			static void deleteCustomer( Auth4cData::Customer& customer );

			static void createDomain( Auth4cData::Domain& domain );
			static void updateDomain( Auth4cData::Domain& domain );
			static void deleteDomain( Auth4cData::Domain& domain );

			static void insertProduct( const Auth4cData::Product& product );
			static void insertProductParam( const Auth4cData::Product& product, const std::string& strProfile, const std::string& strName, const std::string& value );
			static void insertPermission( const Auth4cData::Permission& permission, const Auth4cData::Product& product );
			static void insertRole( const Auth4cData::Role& role, const Auth4cData::Product& product );
			static void insertRolePermission( const Auth4cData::Permission& permission, const Auth4cData::Role& role, const Auth4cData::Product& product );

			static void deleteProduct( const Auth4cData::Product& product, bool bCascade = false );
			static void deleteProductParam( const Auth4cData::Product& product, const std::string& strProfile, const std::string& strName, bool bCascade = false );
			static void deletePermission( const Auth4cData::Permission& permission, bool bCascade = false );
			static void deleteRole( const Auth4cData::Role& role, bool bCascade = false );
			static void deleteRolePermission( const Auth4cData::Permission& permission, bool bCascade = false );

			static int nextPermission( Auth4cData::Product& product );
			static int nextRole( Auth4cData::Product& product );
			static int nextProduct();

			static Base4c::Version version();

	};
};

#endif /* !__Auth4cDB_Auth4cFactory_h__ */
