/*

  $Id: Auth4cFactory.h 688 2018-11-28 17:10:21Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __Auth4cDataXML_Auth4cFactory_h__
#define __Auth4cDataXML_Auth4cFactory_h__

#include <Auth4cDataXML/Auth4cDataXML.h>

#include <Auth4cData/AuthenticatedUser.h>
#include <Auth4cData/Customer.h>
#include <Auth4cData/Profile.h>
#include <Auth4cData/Domain.h>
#include <Auth4cData/Exception.h>
#include <Auth4cData/Permission.h>
#include <Auth4cData/Product.h>
#include <Auth4cData/Role.h>
#include <Auth4cData/Team.h>
#include <Auth4cData/User.h>
#include <Auth4cData/AuthenticationToken.h>
#include <Auth4cData/SessionString.h>
#include <Auth4cData/InitMap.h>

#include <Base4c/Version.h>

namespace Rfc
{
	class XMLNode;
}

namespace Auth4cDataXML
{
	class Auth4cDataXML_Export Factory
	{
		private:
			static Auth4cData::CustomerMap _customerS;
			static Auth4cData::DomainMap _domainS;
			static Auth4cData::ProfileMap _profileS;
			static Auth4cData::TeamMap _teamS;
			static Auth4cData::ProductMap _productS;

		public:
			static std::map<std::string, Base4c::Version> about();
			static void reinitialize();
			static void initialize( const std::string& strInit );
			static Auth4cData::AuthenticatedUser& authenticate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken, const Auth4cData::Profile& profile );
			static Auth4cData::AuthenticatedUser& validate( const Auth4cData::User& user, const Auth4cData::AuthenticationToken& authToken );
			static Auth4cData::AuthenticatedUser& authenticatedUser();
			static bool passwd( const Auth4cData::AuthenticatedUser& user, Auth4cData::AuthenticationToken& authTokenCurrent, Auth4cData::AuthenticationToken& authTokenNew );
			static void persistSession( Auth4cData::AuthenticatedUser& userP );
			static Auth4cData::AuthenticatedUser& retrieveSession( const Auth4cData::Session& session );
			static void killSession( const Auth4cData::Session& session );
			static void user( Auth4cData::User& userF, bool bRecurse = true );
			static void team( Auth4cData::Team& team, bool bReload );
			static Auth4cData::TeamMap& teams();
			static Auth4cData::ProductMap& products();
			static Auth4cData::DomainMap& domains();
			static Auth4cData::CustomerMap& customers();
			static Auth4cData::ProfileMap& profiles();
			static Auth4cData::AuthenticatedUserMap who();

			static Base4c::Version version();

			class Auth4cDataXML_Export Exception
			{
				public:
					static Auth4cData::Exception fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::Exception& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export AuthenticatedUser
			{
				public:
					static Auth4cData::AuthenticatedUser fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::AuthenticatedUser& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export AuthenticatedUserMap
			{
				public:
					static Auth4cData::AuthenticatedUserMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::AuthenticatedUserMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export User
			{
				public:
					static Auth4cData::User fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::User& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export UserMap
			{
				public:
					static Auth4cData::UserMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::UserMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export Session
			{
				public:
					static Auth4cData::Session fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::Session& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export SessionString : public Session
			{
				public:
					static Auth4cData::SessionString fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::SessionString& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export Profile
			{
				public:
					static Auth4cData::Profile fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::Profile& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export ProfileMap
			{
				public:
					static Auth4cData::ProfileMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::ProfileMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export Domain
			{
				public:
					static Auth4cData::Domain fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::Domain& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export DomainMap
			{
				public:
					static Auth4cData::DomainMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::DomainMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export DomainPtrMap
			{
				public:
					static Auth4cData::DomainPtrMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::DomainPtrMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export Customer
			{
				public:
					static Auth4cData::Customer fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::Customer& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export CustomerMap
			{
				public:
					static Auth4cData::CustomerMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::CustomerMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export Team
			{
				public:
					static Auth4cData::Team fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::Team& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export TeamMap
			{
				public:
					static Auth4cData::TeamMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::TeamMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export TeamPtrMap
			{
				public:
					static Auth4cData::TeamPtrMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::TeamPtrMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export TeamMember
			{
				public:
					static Auth4cData::TeamMember fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::TeamMember& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export TeamMemberMap
			{
				public:
					static Auth4cData::TeamMemberMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::TeamMemberMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export Permission
			{
				public:
					static Auth4cData::Permission fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::Permission& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export PermissionMap
			{
				public:
					static Auth4cData::PermissionMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::PermissionMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export Product
			{
				public:
					static Auth4cData::Product fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::Product& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export ProductMap
			{
				public:
					static Auth4cData::ProductMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::ProductMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export ProductInitMap
			{
				public:
					static Auth4cData::ProductInitMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::ProductInitMap& in, bool bRecursive = false );
			};


			class Auth4cDataXML_Export Role
			{
				public:
					static Auth4cData::Role fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::Role& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export RoleMap
			{
				public:
					static Auth4cData::RoleMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::RoleMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export InitMap
			{
				public:
					static Auth4cData::InitMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::InitMap& in, bool bRecursive = false );
			};

			class Auth4cDataXML_Export InitMapMap
			{
				public:
					static Auth4cData::InitMapMap fromXML( const Rfc::XMLNode* pnode );
					static void toXML( Rfc::XMLNode* pnode, const Auth4cData::InitMapMap& in, bool bRecursive = false );
			};
	};
};

#endif /* !__Auth4cDataXML_Auth4cFactory_h__ */
