/*

  $Id: TreeWidgetItem.h 403 2015-12-10 14:13:42Z john $

  Copyright (c) 2008, The 4c Company. All rights reserved.
*/

#ifndef __TreeWidgetItem_h__
#define __TreeWidgetItem_h__

#include <Auth4cUIComponents/auth4cuicomponents_global.h>

#include <QTreeWidget>

#include <Auth4cData/Product.h>
#include <Auth4cData/Domain.h>
#include <Auth4cData/Customer.h>
#include <Auth4cData/Team.h>
#include <Auth4cData/User.h>

namespace Auth4cUIComponents
{
	class AUTH4CUICOMPONENTS_EXPORT TreeWidgetItem : public QTreeWidgetItem
	{
		public:
			typedef enum { Uncheckable = -1, Unchecked, PartiallyChecked, Checked } CheckState;

		private:
			Auth4cData::Base* _pbase;
			CheckState _state;

		public:
			TreeWidgetItem( QTreeWidget* pparent, Auth4cData::Base* pbase, CheckState state = Uncheckable );
			TreeWidgetItem( TreeWidgetItem* pparent, Auth4cData::Base* pbase, CheckState state = Uncheckable );
			virtual ~TreeWidgetItem();

			CheckState state( bool bForReal = true ) const;
			void setState( CheckState state, bool bForReal = true );

			Auth4cData::Base* base();
			virtual void refresh();
			virtual void doMenu();
			virtual void expanded();
	};

	class AUTH4CUICOMPONENTS_EXPORT ProductTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::Product* _pproduct;

		public:
			ProductTreeWidgetItem( QTreeWidget* pparent, Auth4cData::Product* pproduct, CheckState state = Uncheckable );
			virtual ~ProductTreeWidgetItem();

			virtual void refresh();
			virtual void doMenu();
	};

	class AUTH4CUICOMPONENTS_EXPORT PermissionRootTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::Product* _pproduct;

		public:
			PermissionRootTreeWidgetItem( TreeWidgetItem* pparent, Auth4cData::Product* pproduct, CheckState state = Uncheckable );
			virtual ~PermissionRootTreeWidgetItem();

			virtual void refresh();
			virtual void doMenu();
	};

	class AUTH4CUICOMPONENTS_EXPORT PermissionTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::Permission* _ppermission;

		public:
			PermissionTreeWidgetItem( TreeWidgetItem* pparent, Auth4cData::Permission* ppermission, CheckState state = Uncheckable );
			virtual ~PermissionTreeWidgetItem();

			virtual void refresh();
			virtual void doMenu();
	};

	class AUTH4CUICOMPONENTS_EXPORT RoleRootTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::Product* _pproduct;

		public:
			RoleRootTreeWidgetItem( TreeWidgetItem* pparent, Auth4cData::Product* pproduct, CheckState state = Uncheckable );
			virtual ~RoleRootTreeWidgetItem();

			virtual void refresh();
			virtual void doMenu();
	};

	class AUTH4CUICOMPONENTS_EXPORT RoleTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::Role* _prole;

		public:
			RoleTreeWidgetItem( TreeWidgetItem* pparent, Auth4cData::Role* prole, CheckState state = Uncheckable );
			virtual ~RoleTreeWidgetItem();

			virtual void refresh();
			virtual void doMenu();
	};

	class AUTH4CUICOMPONENTS_EXPORT RolePermissionTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::Permission* _ppermission;

		public:
			RolePermissionTreeWidgetItem( TreeWidgetItem* pparent, Auth4cData::Permission* ppermission, CheckState state = Uncheckable );
			virtual ~RolePermissionTreeWidgetItem();

			virtual void refresh();
			virtual void doMenu();
	};

	class AUTH4CUICOMPONENTS_EXPORT CustomerTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::Customer* _pcustomer;

		public:
			CustomerTreeWidgetItem( QTreeWidget* pparent, Auth4cData::Customer* pcustomer, CheckState state = Uncheckable );
			virtual ~CustomerTreeWidgetItem();

			virtual void refresh();
			virtual void doMenu();
	};

	class AUTH4CUICOMPONENTS_EXPORT DomainTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::Domain* _pdomain;

		public:
			DomainTreeWidgetItem( QTreeWidget* pparent, Auth4cData::Domain* pdomain, CheckState state = Uncheckable );
			virtual ~DomainTreeWidgetItem();

			virtual void refresh();
			virtual void doMenu();
	};

	class AUTH4CUICOMPONENTS_EXPORT TeamTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::Team* _pteam;

		public:
			TeamTreeWidgetItem( QTreeWidget* pparent, Auth4cData::Team* pteam, CheckState state = Uncheckable );
			TeamTreeWidgetItem( TreeWidgetItem* pparent, Auth4cData::Team* pteam, CheckState state = Uncheckable );
			virtual ~TeamTreeWidgetItem();

			Auth4cData::Team* team();

			virtual void refresh();
			virtual void doMenu();
			virtual void expanded();
	};

	class AUTH4CUICOMPONENTS_EXPORT TeamMemberRootTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::Team* _pteam;

		public:
			TeamMemberRootTreeWidgetItem( TreeWidgetItem* pparent, Auth4cData::Team* pteam, CheckState state = Uncheckable );
			virtual ~TeamMemberRootTreeWidgetItem();

			virtual void refresh();
			virtual void doMenu();
	};

	class AUTH4CUICOMPONENTS_EXPORT UserTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::User* _puser;

		public:
			UserTreeWidgetItem( TreeWidgetItem* pparent, Auth4cData::User* puser, CheckState state = Uncheckable );
			virtual ~UserTreeWidgetItem();

			Auth4cData::User* user();

			virtual void refresh();
			virtual void doMenu();
	};

	class AUTH4CUICOMPONENTS_EXPORT TeamMemberTreeWidgetItem : public TreeWidgetItem
	{
		private:
			Auth4cData::TeamMember _member;

		public:
			TeamMemberTreeWidgetItem( TreeWidgetItem* pparent, const Auth4cData::TeamMember& member, CheckState state = Uncheckable );
			virtual ~TeamMemberTreeWidgetItem();

			Auth4cData::TeamMember* member();

			virtual void refresh();
			virtual void doMenu();
	};
};

#endif /* !__TreeWidgetItem_h__ */
